package Voyager.db.clients;

import org.omg.CORBA.*;
import org.omg.CosNaming.*;
import org.omg.CosNaming.NamingContextPackage.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.net.*;
import Voyager.db.*;
import Voyager.rtsp.clients.*;
import Voyager.rtsp.*;
import Voyager.util.*;
import java.awt.Frame;

public class VoyagerSDB extends Panel implements ActionListener {

  private org.omg.CORBA.ORB orb;
  private org.omg.CORBA.BOA boa;
  private Voyager.db.VOY_Database db;
  private Voyager.db.SessionDescriptor[] sessionDesc_;
  private URL url_ = null;
  private int demo_ = 0;
  private String ior_ = null;
  java.awt.Label label;
  java.awt.Button button;
  java.awt.Button button1;
  java.awt.Button button2;
  java.awt.Button button3;
  java.awt.Button refreshButton_;
  java.awt.Button demoButton_;
  java.awt.Canvas canvas;
  java.awt.List sessionList_;

  private VoyContext context_;

  public void init( VoyContext context )
  {
    context_ = context;

    context_.initCorba();

    db = context_.getDB();
    
    initGraphics();

    buildSessionList( );
  }

  public void buildSessionList( )
  {
      //context_.showStatus("Retrieving session list");
    try
      {	    
	sessionDesc_ = db.getSessionList( demo_ );
	//context_.showStatus( "Parsing session list" );
	for(int i = 0; i < sessionDesc_.length; i++)
	  sessionList_.add(sessionDesc_[i].name, i);
	
	sessionList_.select( 0 );
      }
    catch(COMM_FAILURE ex)
      {
	System.err.println(ex.getMessage());
	ex.printStackTrace();
	System.exit(1);
      }
  }
  
  public void actionPerformed(ActionEvent e)
  {
    if(e.getActionCommand().equals("Info"))
      getSessionDescription();
    
    if(e.getActionCommand().equals("Play"))
      playSession();

    if(e.getActionCommand().equals("Record"))
      recordSession();

    if(e.getActionCommand().equals("Quit"))
      quit();

    if( e.getActionCommand( ).equals( "Refresh" ) )
      refreshSessionList( );

    if( e.getActionCommand( ).equals( "Toggle Demo" ) )
      toggleDemoFlag( );
  }

  public void refreshSessionList( )
  {
    sessionList_.removeAll();
    buildSessionList( );
  }

  public void toggleDemoFlag( )
  {
    demo_ = (demo_ == 1) ? 0 : 1;
    refreshSessionList( );
  }

  public void setDemo(int f)
  {
    demo_ = f;
  }

  public void getSessionDescription()
  {
    int index = sessionList_.getSelectedIndex();
    String name = sessionList_.getSelectedItem();
    String Description = db.getDescription(sessionDesc_[index].id);
   	
    //context_.showStatus(Description);
   	
    InfoDialog d = new InfoDialog(new Frame(name), name, Description);
    d.show();
  }

  public void playSession()
  {
    int index = sessionList_.getSelectedIndex();
    //context_.showStatus("Index: " + index);

    String name = sessionList_.getSelectedItem();
    //context_.showStatus("Name: " + name);

    String URL = db.getURL(sessionDesc_[index].id);
    //context_.showStatus("URL: " + URL);

    try {
      RtspURL url = new RtspURL(URL);

      PlaybackManager pm = new PlaybackManager();
      pm.init(context_, url, new Frame("Voyager Playback") );
    } catch (MalformedURLException e) {
      System.err.println(e);
    }
  }
  
  public void recordSession()
  {
    Frame frame = new Frame("Recorder");

    RecordController controller = new RecordController(context_);
    RecordUI ui = new RecordUI(controller , frame, context_ );
    frame.add("Center", ui);

    frame.pack();
    frame.setSize(400, 400);
    frame.setVisible(true);

    //context_.showStatus("Starting Recorder...");
  }  
  
  public void quit()
  {
    context_.shutdown();
    /*
    java.lang.Runtime rt = java.lang.Runtime.getRuntime();
    
    rt.exit(0);
    */
  }

  private void initGraphics()
  {
    setLayout(null);

    setSize(300, 400);
    
    // Initialize label
    add(label = new java.awt.Label());
    label.setText("Voyager Session Directory Browser");
    label.setAlignment(1);
    label.setFont(new Font("Helvetica", java.awt.Font.BOLD,  10));
    label.setLocation(69, 13);
    label.setSize(260, 25);
    label.setVisible(true);
    
    // Initialize button
    add(button = new java.awt.Button());
    button.setLabel("Info");
    button.setFont(new Font("Helvetica", java.awt.Font.BOLD,  10));
    button.setLocation(300, 65);
    button.setSize(57, 26);
    button.setVisible(true);
    
    // Initialize button1
    add(button1 = new java.awt.Button());
    button1.setLabel("Play");
    button1.setFont(new Font("Helvetica", java.awt.Font.BOLD,  10));
    button1.setLocation(300, 100);
    button1.setSize(57, 26);
    button1.setVisible(true);
    
    // Initialize button2
    add(button2 = new java.awt.Button());
    button2.setLabel("Record");
    button2.setFont(new Font("Helvetica", java.awt.Font.BOLD,  10));
    button2.setLocation(300, 135);
    button2.setSize(57, 26);
    button2.setVisible(true);
    //    button2.disable();
    
    // Initialize button3
    add(button3 = new java.awt.Button());
    button3.setLabel("Quit");
    button3.setFont(new Font("Helvetica", java.awt.Font.BOLD, 10));
    button3.setLocation(300, 170);
    button3.setSize(57, 26);
    button3.setVisible(true);
    
    // Initialize button3
    add( refreshButton_ = new java.awt.Button() );
    refreshButton_.setLabel("Refresh");
    refreshButton_.setFont(new Font("Helvetica", java.awt.Font.BOLD, 10));
    refreshButton_.setLocation(35, 315);
    refreshButton_.setSize(90, 26);
    refreshButton_.setVisible(true);
    
    // Initialize button3
    add( demoButton_ = new java.awt.Button() );
    demoButton_.setLabel( "Toggle Demo" );
    demoButton_.setFont(new Font("Helvetica", java.awt.Font.BOLD, 9));
    demoButton_.setLocation( 135 , 315 );
    demoButton_.setSize(90, 26);
    demoButton_.setVisible(true);
    
    // Initialize canvas
    add(canvas = new java.awt.Canvas());
    canvas.setLocation(309, 184);
    canvas.setSize(106, 128);
    canvas.setVisible(true);
    
    // Initialize list
    add(sessionList_ = new java.awt.List());
    sessionList_.setFont(new Font("Helvetica", java.awt.Font.PLAIN, 10));
    sessionList_.setLocation(12, 52);
    sessionList_.setSize(250, 253);
    sessionList_.setVisible(true);
    
    // Initialize adaptor
    button.addActionListener(this);
    button1.addActionListener(this);
    button2.addActionListener(this);
    button3.addActionListener(this);
    refreshButton_.addActionListener( this );
    demoButton_.addActionListener( this );
  }
}

