package Voyager.rtsp;

import java.util.*;
import java.io.*;

class Request
{
    Request(String method, String uri)
    {
	method_ = method;
	uri_ = uri;
	sequence_ = nextSequence_++;
	response_ = null;
	headers_  = new Hashtable();
	body_ = null;
    }

    String method() { return method_; }
    String uri() { return uri_; }
    String version() { return version_; }
    int sequence() { return sequence_; }
    public Integer sequenceObj() { return new Integer(sequence_); }

    void setHeader(String header, String value)
    {
	headers_.put(header, value);
    }
    
    void setBody(byte b[])
    {
	body_ = b;
    }

    public void send(PrintStream stream)
    {
	String line = method() + " " + uri() + " " + version() + " " + sequence();
//	System.out.println("line is " + line);

	stream.println(line);

	Enumeration e = headers_.keys();
	while (e.hasMoreElements())
	{
	    String key = (String) e.nextElement();
	    String value = (String) headers_.get(key);
	    stream.println(key + ": " + value);
	}
	stream.println("");
    }

    //
    // Wait until we're signalled, then return the response
    // that was filled in.
    synchronized public Response waitForResponse()
    {
	while (response_ == null)
	{
	    try {
		wait();
	    }
	    catch (InterruptedException ex)
	    {
	    }
	}
	return response_;
    }

    //
    // Hey, we got it. Set the field & notify ourself.
    //
    synchronized void setResponse(Response response)
    {
	response_ = response;
	notifyAll();
    }

    Response response_;
    String method_;
    String uri_;
    int sequence_;

    Hashtable headers_;
    byte body_[];

    final String version_ = "RTSP/0";
    static int nextSequence_ = 100;
}
