package Voyager.rtsp;

import java.net.*;
import java.io.*;
import java.util.*;

public class TCPHandler implements Runnable
{
    public TCPHandler(String host, int port) throws UnknownHostException, IOException
    {
	host_ = host;
	port_ = port;
	socket_ = new Socket(host, port);

	inputStream_ = new DataInputStream(socket_.getInputStream());
	outputStream_ = new PrintStream(socket_.getOutputStream());

	requestTable_ = new Hashtable();
    }

    public void run()
    {
	System.out.println("TCPHandler " + host_ + "/" + port_ + ": run()");

	try
	{
	    String line;
	    boolean done = false;

	    //
	    // Read a line first, hack here. Well, maybe not, if we
	    // keep a greeting message in the server.
	    //
//	    line = inputStream_.readLine();
//	    System.out.println("read line " + line);
	    
	    while (!done)
	    {
//		System.out.println("reading line");
		line = inputStream_.readLine();
//		System.out.println("read line " + line);

		if (line == null)
		{
//		    System.out.println("Got EOF");
		    done = true;
		}
		else
		{
		    done = handleInput(line);
		}
	    }
	}
	catch (IOException iox)
	{
	    System.out.println("IO Exception " + iox);
	}
    }

    boolean handleInput(String line)
    {
//	System.out.println("Handling input " + line);

	StringTokenizer tokenizer = new StringTokenizer(line);

	//
	//      Status-Line = RTSP-Version SP Status-Code SP seq-no SP Reason-Phrase CRLF
	//      Request-Line = Method SP Request-URI SP RTSP-Version SP seq-no CRLF
	//
	// See if this is a request or a response. Both have four elements. Distinguish
	// based on the first. If it has a slash, it's an RTSP-Version which means it
	// is a response.
	//

	String el1, el2, el3, el4;

	el1 = el2 = el3 = el4 = null;

	if (tokenizer.hasMoreTokens())
	{
	    el1 = tokenizer.nextToken();
	}
	if (tokenizer.hasMoreTokens())
	{
	    el2 = tokenizer.nextToken();
	}
	if (tokenizer.hasMoreTokens())
	{
	    el3 = tokenizer.nextToken();
	}
	if (tokenizer.hasMoreTokens())
	{
	    el4 = tokenizer.nextToken();
	}

	if (el4 == null)
	{
	    System.out.println("Invalid input " + line);
	    return true;
	}

	if (el1.indexOf("/") == -1)
	{
	    // It's a request
	    return handleRequest(el1, el2, el3, Integer.parseInt(el4));
	}
	else
	{
	    // It's a response
	    boolean rc;
	    try {
		rc = handleResponse(el1,
				    Integer.parseInt(el2),
				    Integer.parseInt(el3),
				    el4);
	    }
	    catch (Exception ex)
	    {
		System.out.println(ex + "> <" + el1 + "> <" + el2 + "> <" + el3 + "> <" + el4);
		rc = true;
	    }
	    return rc;
	}
    }

    boolean handleRequest(String method, String uri, String version, int seq)
    {
	//System.out.println("handleRequest " + method + " " + uri + " " + version + " " + seq);
	return false;
    }

    //
    // Read the response text from the network.
    // Construct a Response object
    // Look up the request by the sequence number in the response,
    // and call setResponse on it. This wakes up the requesting thread.
    //
    
    boolean handleResponse(String version, int statusCode, int sequence, String reason)
    {
//	System.out.println("handleResponse " + version + " " + statusCode + " " + sequence + " " + reason);
	Hashtable headers = readHeaders();
	byte data[];

	data = null;

//	System.out.println("got headers, size=" + headers.size());
	Enumeration e = headers.keys();
	while (e.hasMoreElements())
	{
	    String key = (String) e.nextElement();
	    String value = (String) headers.get(key);
//	    System.out.println(key + ": " + value);
	}

	String slen = (String) headers.get("Content-Length");
	if (slen != null)
	{
	    int len = Integer.parseInt(slen);
//	    System.out.println("getting body len=" + len);
	    data = new byte[len];
	    try {
		inputStream_.readFully(data);
	    } catch (Exception ex) {}
	    String datastr = new String(data, 0);
	    //System.out.println("Got body " + datastr);
	}

	Request request;
	Response response;

	response = new Response(version, statusCode, sequence, reason);
	response.setBody(data);
	response.setHeaders(headers);

	request = (Request) requestTable_.get(response.sequenceObj());
	request.setResponse(response);
	requestTable_.remove(response.sequenceObj());

	return false;
    }

    Hashtable readHeaders()
    {
	String line;
	Hashtable rc = new Hashtable();

	while (true)
	{
	    line = null;
	    try {
		line = inputStream_.readLine();
	    }
	    catch (Exception ex) {}
//	    System.out.println("read line <" + line + ">");

	    if (line == null)
	    {
		System.out.println("Got EOF");
		break;
	    }
	    else if (line.equals(""))
	    {
//		System.out.println("got empty line");
		break;
	    }
	    else
	    {
		StringTokenizer st = new StringTokenizer(line, ": ");
		String hdr, value;
		hdr = value = null;
		if (st.hasMoreTokens())
		{
		    hdr = st.nextToken();
		}
		if (st.hasMoreTokens())
		{
		    value = st.nextToken();
		}
//		System.out.println("Got header: " + hdr + " " + value);
		if (value == null)
		{
		    System.out.println("Invalid header line " + line);
		    return null;
		}
		else
		{
		    rc.put(hdr, value);
		}
	    }
	}
	return rc;
    }

    //
    // Send a request. After sending the request, we need to 
    // wait for the reply, which is processed by the handler
    // thread. To keep requests and replies synched, we
    // register the request in the handler's requestTable_
    // and wait on the request object.
    //
    Response sendRequest(Request request)
    {
	Response response;

	synchronized(this)
	{
	    request.send(outputStream_);
	}

	requestTable_.put(request.sequenceObj(), request);

	System.out.println("Sent request, waiting for response");
	response = request.waitForResponse();
	System.out.println("Got response " + response.code() + " " + response.sequence());
	return response;
    }

    String host_;
    int port_;
    Socket socket_;
    DataInputStream inputStream_;
    PrintStream outputStream_;
    Hashtable requestTable_;
}
