package Voyager.rtsp.clients;

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.text.*;
import java.lang.*;

public class DestPortUI extends Dialog implements ActionListener
{
    TextField[]   destinations;
    TextField[]   ports;
    TextField     ttl;
    Label[]       destLabs;
    Label[]       portLabs;
    Label         ttlLab;
    Label         destFormat;
    Label         portFormat;
    
    boolean qChanged_;

    Button  cancelButton_;
    Button  okButton_;
    Button  resetButton_;

    DestPortData  dpData_;

    boolean  qVideo_;
    boolean  qAudio_;
    boolean  qVr_;

    public boolean dataChanged( ) { return qChanged_; }

    public DestPortUI( Frame parent , DestPortData data , boolean qVideo ,
		       boolean qAudio , boolean qVr )
    {
	// make it modal
	super( parent , "Playback Destination and Port" , true );

	dpData_ = data;

	qVideo_ = qVideo;
	qAudio_ = qAudio;
	qVr_ = qVr;
	
	qChanged_ = false;

	resetDestPortControls( );
    }

    public void actionPerformed( ActionEvent e )
    {
	if( e.getActionCommand( ).equals( "OK" ) )
	    processOkCmd( );
	else if( e.getActionCommand( ).equals( "Cancel" ) )
	    processCancelCmd( );
	else if( e.getActionCommand( ).equals( "Reset" ) )
	    processResetCmd( );
    }

    private void processOkCmd( )
    {
	int rc = saveDestPortControls( );
	if( rc > 0 )
	{
	    // Error on save.  Require user to fix it.
	    switch( rc )
	    {
	    case 1:
		destinations[0].requestFocus( );
		break;
	    case 2:
		destinations[1].requestFocus( );
		break;
	    case 3:
		destinations[2].requestFocus( );
		break;
	    case 4:
		ports[ 0 ].requestFocus( );
		break;
	    case 5:
		ports[ 1 ].requestFocus( );
		break;
	    case 6:
		ports[ 2 ].requestFocus( );
		break;
	    case 7:
		ttl.requestFocus( );
		break;
	    }
	}
	else
	{
	    qChanged_ = true;
	    invalidate( );
	    dispose( );
	}
    }

    private void processCancelCmd( )
    {
	qChanged_ = false;
	invalidate( );
	dispose( );
    }
    
    private void processResetCmd( )
    {
	qChanged_ = false;
	resetDestPortControls( );
	validate( );
    }
    
    public DestPortData destPortData( )
    {
	return dpData_;
    }

    public void resetDestPortControls( )
    {
	removeAll( );

	destFormat = new Label( "xxx.xxx.xxx.xxx" );
	portFormat = new Label( "integers only" );
	
	destinations = new TextField[ 3 ];
	destinations[ 0 ] = new TextField( dpData_.audioDest( ) , 15 );
	destinations[ 0 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	destinations[ 1 ] = new TextField( dpData_.videoDest( ) , 15 );
	destinations[ 1 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	destinations[ 2 ] = new TextField( dpData_.vrDest( ) , 15 );
	destinations[ 2 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	destLabs = new Label[ 3 ];
	destLabs[ 0 ] = new Label( "audio destination:" );
	destLabs[ 0 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	destLabs[ 1 ] = new Label( "video destination:" );
	destLabs[ 1 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	destLabs[ 2 ] = new Label( "   vr destination:" );
	destLabs[ 2 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	ports =  new TextField[ 3 ];
	ports[ 0 ] = new TextField( Integer.toString(dpData_.audioPort()),6 );
	ports[ 1 ] = new TextField( Integer.toString(dpData_.videoPort()),6 );
	ports[ 2 ] = new TextField( Integer.toString(dpData_.vrPort()),6 );
	portLabs = new Label[ 3 ];
	portLabs[ 0 ] = new Label( "audio port:" );
	portLabs[ 0 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	portLabs[ 1 ] = new Label( "video port:" );
	portLabs[ 1 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	portLabs[ 2 ] = new Label( "   vr port:" );
	portLabs[ 2 ].setFont(new Font("Helvetica", Font.BOLD, 12));
	ttl = new TextField( Integer.toString( dpData_.ttl( )) , 3 );
	ttlLab = new Label( "ttl:" , Label.RIGHT );
	ttlLab.setFont(new Font("Helvetica", Font.BOLD, 12));

	Panel destPortPanel = new Panel( );
	GridBagLayout gb = new GridBagLayout( );
	destPortPanel.setLayout( gb );
	GridBagConstraints c = new GridBagConstraints( );

	c.gridy = 0;
	c.gridx = 1;
	gb.setConstraints( destFormat , c );
	destPortPanel.add( destFormat );

	c.gridx = 3;
	gb.setConstraints( portFormat , c );
	destPortPanel.add( portFormat );

	for( int i = 0 ; i < 3 ; i++ )
	{
	    c.gridy = i+1;

	    if( (qAudio_ && i==0) ||
		(qVideo_ && i==1) ||
		(qVr_ && i==2 ) )
	    {
		c.gridx = 0;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints( destLabs[ i ] , c );
		destPortPanel.add( destLabs[ i ] );

		c.gridx = 1;
		c.anchor = GridBagConstraints.WEST;
		gb.setConstraints( destinations[ i ] , c );
		destPortPanel.add( destinations[ i ] );

		c.gridx = 2;
		c.anchor = GridBagConstraints.EAST;
		gb.setConstraints( portLabs[ i ] , c );
		destPortPanel.add( portLabs[ i ] );

		c.gridx = 3;
		c.anchor = GridBagConstraints.WEST;
		gb.setConstraints( ports[ i ] , c );
		destPortPanel.add( ports[ i ] );
	    }
	}

	c.fill = GridBagConstraints.NONE;
	c.anchor = GridBagConstraints.EAST;
	c.gridy = 4;
	c.gridx = 1;
	gb.setConstraints( ttlLab , c );
	destPortPanel.add( ttlLab );

	c.anchor = GridBagConstraints.WEST;
	c.gridx = 2;
	gb.setConstraints( ttl , c );
	destPortPanel.add( ttl );

	Panel controlPanel = new Panel( );
	controlPanel.setLayout( new FlowLayout( FlowLayout.CENTER , 5 , 5 ) );

	cancelButton_ = new Button( "Cancel" );
	cancelButton_.addActionListener( this );
	okButton_ = new Button( "OK" );
	okButton_.addActionListener( this );
	resetButton_ = new Button( "Reset" );
	resetButton_.addActionListener( this );
	controlPanel.add( okButton_ );
	controlPanel.add( cancelButton_ );
	controlPanel.add( resetButton_ );

	setLayout( new BorderLayout( ) );
	add( "Center" , destPortPanel );
	add( "South" , controlPanel );

	pack( );
    }

    private int saveDestPortControls( )
    {
	int rc = 0;
	int audioPort, videoPort, vrPort, ttlInt;
	audioPort = videoPort = vrPort = ttlInt = 0;

	// Before saving, check all the values
	//
	if( qAudio_ && !dpData_.audioDestOk(destinations[0].getText()) )
	    rc = 1;
	else if( qVideo_ && !dpData_.videoDestOk(destinations[1].getText()) )
	    rc = 2;
	else if( qVr_ && !dpData_.vrDestOk(destinations[2].getText()) )
	    rc = 3;
	else if( qAudio_ )
	{
	    try {
		audioPort = Integer.parseInt( ports[ 0 ].getText( ) );
		if( !dpData_.audioPortOk(  audioPort ) )
		    rc = 4;
	    } catch( java.lang.NumberFormatException e ) {
		rc = 4;
	    }
	}
	if( rc == 0 && qVideo_ )
	{
	    try {
		videoPort =  Integer.parseInt( ports[ 1 ].getText( ) );
		if( !dpData_.videoPortOk( videoPort ) )
		    rc = 5;
	    } catch( java.lang.NumberFormatException e ) {
		rc = 5;
	    }
	}
	if( rc == 0 && qVr_ )
	{
	    try {
		vrPort = Integer.parseInt( ports[2 ].getText( ) );
		if( !dpData_.vrPortOk( vrPort ) )
		    rc = 6;
	    } catch( java.lang.NumberFormatException e ) {
		rc = 6;
	    }
	}
	if( rc == 0 )
	{
	    try {
		ttlInt = Integer.parseInt( ttl.getText( ) );
		if( !dpData_.ttlOk( ttlInt ) )
		    rc = 7;
	    } catch( java.lang.NumberFormatException e ) {
		rc = 7;
	    }
	}

	// Now save them if they're all ok
	if( rc == 0 )
	{
	    if( qAudio_ )
	    {
		dpData_.audioDest( destinations[ 0 ].getText( ) );
		dpData_.audioPort(  audioPort );
	    }

	    if( qVideo_ )
	    {
		dpData_.videoDest( destinations[ 1 ].getText( ) );
		dpData_.videoPort( videoPort );
	    }

	    if( qVr_ )
	    {
		dpData_.vrDest( destinations[ 2 ].getText( ) );
		dpData_.vrPort( vrPort );
	    }
	    dpData_.ttl( ttlInt );
	}

	return rc;
    }
}

