// **********************************************************************
//
// Copyright (c) 1997
// Object-Oriented Concepts, Inc.
// Billerica, MA, USA
//
// All Rights Reserved
//
// **********************************************************************

#include <OB/CORBA.h>
#include <OB/Util.h>

#include <VOY_Database.h>

#include <stdlib.h>

#ifdef HAVE_FSTREAM
#   include <fstream>
#else
#   include <fstream.h>
#endif

int
main(int argc, char* argv[], char*[])
{
  SessionDescriptorSeq_var slist;

    try
    {
	//
	// Create ORB
	//
	CORBA_ORB_var orb = CORBA_ORB_init(argc, argv);
	
	//
	// Get "hello" object
	//
	const char* refFile = "VDI.ref";
	ifstream in(refFile);
	if(in.fail())
	{
	    extern int errno;
	    cerr << argv[0] << ": can't open `" << refFile << "': "
		 << strerror(errno) << endl;
	    return 1;
	}
	
	char s[1000];
	in >> s;
	
	CORBA_Object_var obj = orb -> string_to_object(s);
	assert(!CORBA_is_nil(obj));
	
	VOY_Database_var db = VOY_Database::_narrow(obj);
	assert(!CORBA_is_nil(db));

	//
	// Main loop
	//

	slist = db->getSessionList(0);
	cout << slist->length() << " entries\n";
	for(int i = 0; i < slist->length(); i++)
	  {
	    CORBA_String_var name = slist[i].name;
	    CORBA_Long id = slist[i].id;

	    cout << "Session: " << name << " ID: " << id << endl;
	    cout << "\tURL: " << db->getURL(id) << endl;
	    cout << "\tDescription: " << db->getDescription(id) << endl;
	    CORBA_String_var host = CORBA_string_dup("eyas01.mcs.anl.gov");
	    CORBA_String_var ssrc = CORBA_string_dup("foo");
	    
	    CORBA_String_var path = db->getStreamPath(id, host, ssrc);
	    cout << "path is " << path << endl;
	  }
    }
#ifdef __GNUG__
    catch(CORBA_COMM_FAILURE& ex)
#else
    catch(CORBA_SystemException& ex)
#endif
    {
	OBPrintException(ex);
	return 1;
    }
    
    return 0;
}

