#include "Dispatcher.h"

#include "config.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Request.h"
#include "Session_Manager.h"
#include "Playback_Manager.h"
#include "Record_Manager.h"
#include "RTSP_Teardown_Request.h"

Dispatcher::Dispatcher(Session_Manager *s, Playback_Manager *pm,
		       Record_Manager *rm) :
    sessionManager_(s),
    playbackManager_(pm),
    recordManager_(rm)
{
}


void Dispatcher::handleRequestPacket(RTSP_Request_Packet *requestPacket)
{
    printf("Handling request method=%s uri=%s version=%s\n",
	   requestPacket->method(), requestPacket->uri(),
	   requestPacket->version());

    RTSP_Request *request;

    request = RTSP_Request::createRequest(requestPacket);

    //
    // Yeehaw, double dispatch is YER FRIEND.
    //

    if( request )
	request->dispatch(this);
}

void Dispatcher::handleDescribeRequest(RTSP_Describe_Request *req)
{
    sessionManager_->handleDescribe(req);
}

void Dispatcher::handleRecordSetupRequest(RTSP_Setup_Request *req)
{
    recordManager_->handleSetup(req);
}

void Dispatcher::handleRecordRequest(RTSP_Record_Request *req)
{
    recordManager_->handleRecord(req);
}

void Dispatcher::handlePlaySetupRequest(RTSP_Setup_Request *req)
{
    playbackManager_->handleSetup(req);
}

void Dispatcher::handlePlayRequest(RTSP_Play_Request *req)
{
    playbackManager_->handlePlay(req);
}

void Dispatcher::handleTeardownRequest( RTSP_Teardown_Request* request )
{
    if( playbackManager_->isPlaySession( request ) )
	playbackManager_->handleTeardown( request );
    else if( recordManager_->isRecordSession( request ) )
	recordManager_->handleTeardown( request );
}


