#ifndef _Network_Endpoint_h_
#define _Network_Endpoint_h_

#include <ace/INET_Addr.h>

class Network_Endpoint
{
public:
    Network_Endpoint( ) : host_( "" ) , port_( -1 ) , ttl_( -1 ) , any_( 0 )
    {}

    Network_Endpoint( const VOY_String& h , int p , int t = -1 ) : host_( h ) , port_( p ) , ttl_( t ) , any_( 0 )
    {}

    Network_Endpoint( int p ) : host_( "" ) , port_( p ) , ttl_( -1 ) , any_( 1 )
    {}

    Network_Endpoint( const Network_Endpoint& ePoint ) :
	host_( ePoint.host_ ) , port_( ePoint.port_ ) , ttl_( ePoint.ttl_ ) , any_( ePoint.any_ )
    {}

    void set( const VOY_String& h , int p , int t = -1 )
    {
	host_ = h;
	port_ = p;
	ttl_ = t;
	any_ = 0;
    }

    void set( int p )
    {
	host_ = "";
	port_ = p;
	ttl_ = -1;
	any_ = 1;
    }

    bool operator==( const Network_Endpoint& ePoint ) const
    {
	return ePoint.host_ == host_ && ePoint.port_ == port_ && ePoint.ttl_ == ttl_ &&
	    ePoint.any_ == any_;
    }

    bool operator<( const Network_Endpoint& ePoint ) const
    {
	if( host_ < ePoint.host_ )
	    return true;
	else if( host_ > ePoint.host_ )
	    return false;

	if( port_ < ePoint.port_ )
	    return true;
	else if( port_ > ePoint.port_ )
	    return false;

	if( ttl_ < ePoint.ttl_ )
	    return true;
	else if( ttl_ > ePoint.ttl_ )
	    return false;

	return ePoint.any_ != any_;
    }

    bool isMulticast( void )
    {
	if( any_ )
	    return 0;
	ACE_INET_Addr addr( port_ , host_.c_str( ) );

	return IN_CLASSD( addr.get_ip_address( ) );
    }

    ACE_INET_Addr rtpAddress( void )
    {
	if( any_ )
	    return ACE_INET_Addr( port_ );
	else
	    return ACE_INET_Addr( port_ , host_.c_str( ) );
    }

    ACE_INET_Addr rtcpAddress( void )
    {
	if( any_ )
	    return ACE_INET_Addr( port_ + 1 );
	else
	    return ACE_INET_Addr( port_ + 1 , host_.c_str( ) );
    }

    VOY_String host_;
    int port_;
    int ttl_;
    
    // This is a flag that indicates whether or not the endpoint is bound to an address -
    // any_ == 1 if any host can send to it.  For mulicast addresses, any_ is always == 0
    int any_;
};

inline ostream &operator<<( ostream& os , const Network_Endpoint& ePoint )
{
    if( ePoint.any_ )
    {
	os << "*/" << ePoint.port_;
    }
    else
    {
	os << ePoint.host_ << "/" << ePoint.port_ << "/" << ePoint.ttl_;
    }
    return os;
}
    
#endif
