#include "Network_Transport.h"
#include "RTP_Packet.h"

/////////////////////////////////////////////////////////////////////////////
//
// Network_Transport.C
//
// Revision History
//      29 Sep 97 - lisa - original version
//

#include <sys/errno.h>
#include <stream.h>
#include <ace/Reactor.h>

Network_Transport::Network_Transport(EndpointIdentifier ei) :
    endpoint_(ei),
    rtp_address_(ei.rtpAddress()),
    rtcp_address_(ei.rtcpAddress()),
    isMulticast_(ei.isMulticast())
{
    cout << form("Created transport %x for %s/%d\n", this,
	   rtp_address_.get_host_addr(), rtp_address_.get_port_number());
    cout << "endpoint is " << ei << endl;


    if (isMulticast_)
    {
	int rc;
	
	rc = rtp_mc_sock_.subscribe(rtp_address_);
	if (rc < 0)
	    perror("subscribe");
	    
	rc = rtcp_mc_sock_.subscribe(rtcp_address_);
	if (rc < 0)
	    perror("subscribe");
	    
	ei.ttl = 16;
	if (ei.ttl > 1)
	{
	    int h;
	    u_char ttl;

	    /*
	    h = rtp_mc_sock_.get_handle();
	    ttl = ei.ttl;
	    rc = ::setsockopt(h, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
	    if (rc < 0)
		perror("MULTICAST_TTL");

	    h = rtcp_mc_sock_.get_handle();
	    ttl = ei.ttl;
	    rc = ::setsockopt(h, IPPROTO_IP, IP_MULTICAST_TTL, &ttl, sizeof(ttl));
	    if (rc < 0)
		perror("MULTICAST_TTL");
		*/
	    if (rtp_mc_sock_.set_option(IP_MULTICAST_TTL, ei.ttl) < 0)
		perror("rtp set ttl");
	    if (rtcp_mc_sock_.set_option(IP_MULTICAST_TTL, ei.ttl) < 0)
		perror("rtcp set ttl");
	    if (rtp_mc_sock_.set_option(IP_MULTICAST_LOOP, 0) < 0)
		perror("rtp set loop");
	    if (rtcp_mc_sock_.set_option(IP_MULTICAST_LOOP, 0) < 0)
		perror("rtcp set loop");
	    
	}

	//
	// Register for updates
	//

	ACE_Reactor::instance()->register_handler(rtp_mc_sock_.get_handle(),
						this,
						ACE_Event_Handler::READ_MASK);
	ACE_Reactor::instance()->register_handler(rtcp_mc_sock_.get_handle(),
						this,
						ACE_Event_Handler::READ_MASK);
    }
    else
    {
	if (rtp_sock_.open(ACE_INET_Addr(ei.port)) < 0)
	    perror("rtp_sock open");

	if (rtcp_sock_.open(ACE_INET_Addr(ei.port + 1)) < 0)
	    perror("rtcp_sock open");

	ACE_Reactor::instance()->register_handler(rtp_sock_.get_handle(),
						this,
						ACE_Event_Handler::READ_MASK);
	ACE_Reactor::instance()->register_handler(rtcp_sock_.get_handle(),
						this,
						ACE_Event_Handler::READ_MASK);
    }
}

Network_Transport::~Network_Transport( )
{
}

void
Network_Transport::playRtpPacket( int type , char* packet , int len )
{
    //printf( "Network_Transport::playRtpPacket: playing RTP\n" );

    int rc;
    if (isMulticast_)
    {
	rc = rtp_mc_sock_.send((const void *) packet, len);
    }
    else
    {
	rc = rtp_sock_.send((const void *) packet, len, rtp_address_);
    }

    if (rc < 0)
    {
	fprintf(stderr, "send_rtp_packet(): send %d failed: %s\n",
		len, strerror(errno));
    }

}

void
Network_Transport::playRtcpPacket( char* packet , int len )
{
    //printf( "Network_Transport::playRtcpPacket: playing RTCP\n" );
    int rc;

    if (isMulticast_)
    {
	rc = rtcp_mc_sock_.send((const void *) packet, len);
    }
    else
    {
	rc = rtcp_sock_.send((const void *) packet, len, rtcp_address_);
    }

    if (rc < 0)
    {
	cout << form("send_rtcp_packet(): send %d failed: %s\n",
		len, strerror(errno));
	cout << "transport is " << *this << endl;
    }
}

int Network_Transport::handle_input(ACE_HANDLE fd)
{
    if (fd == rtp_sock_.get_handle())
    {
	printf("input on rtp_sock\n");
    }
    else if (fd == rtcp_sock_.get_handle())
    {
	printf("input on rtcp_sock\n");
    }
    else if (fd == rtp_mc_sock_.get_handle())
    {
	printf("input on rtp_mc_sock\n");
    }
    else if (fd == rtcp_mc_sock_.get_handle())
    {
	printf("input on rtcp_mc_sock\n");
    }
    char buf[4096];
    int n = ACE_OS::recv(fd, buf, sizeof(buf));
    printf("Read %d bytes\n", n);
    return 0;
}
