#include "config.h"
#include "Playback_Manager.h"
#include "RTSP_Setup_Request.h"
#include "RTSP_Play_Request.h"
#include "Playback_Network_Transport.h"
#include "Play_Session_Manager.h"
#include "Transport_Manager.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Response.h"
#include "RTSP_Teardown_Request.h"

Playback_Manager::Playback_Manager(Transport_Manager *tm ,
				   VOY_Database_ptr db , 
				   VOY_String payFreqFileName )
                                   : transportManager_(tm),  databaseRef_(db),
                                     nextSessionId_(100) , 
				     payFreqFileName_( payFreqFileName )
{
}

void Playback_Manager::handleSetup(RTSP_Setup_Request *request)
{
    //cout << "Playback_Manager handling setup " << *request << endl;

    Playback_Network_Transport* netTrans;

    netTrans = transportManager_->setupPlaybackTransport( request );

    if( netTrans == NULL )
	throw new RTSP_Response( 450 , "Cannot create playback transport" ,
				 request->requestPacket( )->sequence( ) );

    Play_Session_Manager*  playManager = NULL;
	
    //
    // Poke at the request to see if we've got a session id yet
    //
    const char *session = request->requestPacket()->headers()["Session"];

    //
    // If the request comes in with a session id already, try
    // to find that session. If it's not there, throw an exception
    // (client evidently sent a bad id. Clients can't choose ids).
    //
    // If there wasn't a session id already, create a new one.
    //

    //cout << "Creating session stuff\n";

    Play_Session_ID sessionId;

    if (session != 0 && session[0] != '\0')
    {
	//cout << "Looking up session " << session << endl;

	Play_Session_Map::iterator iter;

	sessionId = atoi(session);
	
	iter = playSessionMap_.find(sessionId);
	if (iter != playSessionMap_.end())
	{
	    playManager = (*iter).second;
	    //cout << "Found manager " << playManager << " in map\n";
	}
	else
	{
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
	}
    }
    else
    {
 	Play_Session_ID myid(nextSessionId_++);

	playManager = new Play_Session_Manager(databaseRef_, myid , payFreqFileName_ );

	//cout << "Creating new play_session_mgr " << playManager << "\n";

	playSessionMap_[myid] = playManager;
    }

    if(  !(playManager   &&
	   playManager->addStream( request , netTrans )==Play_Session_Manager::kSuccess )  )
    {
	// throw exception
    }
}

void Playback_Manager::handlePlay(RTSP_Play_Request *request)
{
    //
    // Poke at the request to see if we've got a session id yet
    //

    const char *session = request->requestPacket()->headers()["Session"];

    Play_Session_ID sessionId;
    Play_Session_Manager *playManager;

    if (session != 0 && session[0] != '\0')
    {
	//cout << "Looking up session " << session << endl;

	Play_Session_Map::iterator iter;

	sessionId = atoi(session);
	
	iter = playSessionMap_.find(sessionId);
	if (iter != playSessionMap_.end())
	{
	    playManager = (*iter).second;
	    //cout << "Found manager " << playManager << " in map\n";
	}
	else
	{
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
	}
    }
    else
    {
	throw new RTSP_Response(450, "Invalid session id",
				request->requestPacket()->sequence());
    }

    cout << "playManager " << playManager << " gets play request for id ";
    cout << sessionId << endl;

    playManager->play( request );
}

void Playback_Manager::handleTeardown( RTSP_Teardown_Request* request )
{
    //cout << "Playback_Manager handling TEARDOWN " << *request << endl;

    const char *session = request->requestPacket()->headers( )["Session"];

    Play_Session_ID               sessionId;
    Play_Session_Manager* playManager;

    if( session != 0  &&  session[0] != '\0')
    {
	//cout << "Looking up session " << session << endl;

	Play_Session_Map::iterator iter;

	sessionId = atoi( session );
	
	iter = playSessionMap_.find( sessionId );

	// Hand the request down to the session manager.  If this is the last stream in the
	// session, delete the Play_Session_Manager
	if( iter != playSessionMap_.end( ) )
	{
	    playManager = (*iter).second;
	    //cout << "Found manager " << playManager << " in map\n";
	    //cout << "playManager " << playManager << " gets teardown request for id ";
	    //cout << sessionId << endl;

	    short qNoMoreStreams;
	    playManager->teardown( request , qNoMoreStreams );
	    if( qNoMoreStreams )
	    {
		// printf( "Playback_Manager::teardown: No more streams in Play_Session_Manager - deleting playManager\n" );
		// Why does deleting the playManager cause db->getStreamPath( )
		// in Stream_Source to seg fault when the next session is setup?
		//delete playManager; 
		playSessionMap_.erase( sessionId );
	    }
	}
	else
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
    }
    else
	throw new RTSP_Response(450, "Invalid session id",
				request->requestPacket()->sequence());
}


short
Playback_Manager::isPlaySession( RTSP_Teardown_Request* request )
{
    short isPlaySession = 0;
    
    const char *session = request->requestPacket()->headers( )["Session"];

    if( session != 0  &&  session[0] != '\0')
    {
	Play_Session_ID               sessionId;
	sessionId = atoi( session );
	
	Play_Session_Map::iterator iter;
	iter = playSessionMap_.find( sessionId );

	isPlaySession =  ( iter != playSessionMap_.end( ) );
    }

    //printf( "Playback_Manager::isPlaySession returns %d\n" , isPlaySession );
    
    return isPlaySession;
}
