#include "RTSP_Setup_Request.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Transport_Info.h"
#include "RTSP_Response.h"
#include "Dispatcher.h"

RTSP_Setup_Request::RTSP_Setup_Request() :
    serverSessionId_(-1)
{
}

int RTSP_Setup_Request::setRequestPacket(RTSP_Request_Packet *r)
{
    //cout << "Set request\n";

    if (parseTransport(r) < 0)
	return -1;

    if (parseSession(r) < 0)
	return -1;

    requestPacket_ = r;
    return 0;
}    

void RTSP_Setup_Request::dispatch(Dispatcher *d)
{
    if (isModePlay())
    {
	d->handlePlaySetupRequest(this);
    }
    else if (isModeRecord())
    {
	d->handleRecordSetupRequest(this);
    }
}

int RTSP_Setup_Request::parseTransport(RTSP_Request_Packet *r)
{
    const char *transport = r->headers()["Transport"];
    //cout << "Transport is " << transport << endl;

    //
    // Transport is required for setup.
    //

    if (transport[0] == '\0')
	throw new RTSP_Response(459, "Header field not valid for resource",
				r->sequence());

    transportInfo_ = new RTSP_Transport_Info();
    if (transportInfo_->parse(transport) < 0)
    {
	throw new RTSP_Response(459, "Header field not valid for resource",
				r->sequence());
    }
    return 0;
}

int RTSP_Setup_Request::parseSession(RTSP_Request_Packet *r)
{
    const char *session = r->headers()["Session"];
    if (session[0] != '\0')
    {
	int id = atoi(session);

	if (id == 0)
	{
	    throw new RTSP_Response(459, "Header field not valid for resource",
				    r->sequence());
	}

	serverSessionId_ = id;
    }

    return 0;
}

int RTSP_Setup_Request::isModePlay(void) const
{
    return transportInfo_->isPlay();
}

int RTSP_Setup_Request::isModeRecord(void) const
{
    return transportInfo_->isRecord();
}

ostream& RTSP_Setup_Request::put(ostream &os) const
{
    os << "RTSP_Setup_Request:\n";
    os << "streamId: " << streamId_ << endl;
    os << "sessionId: " << sessionId_ << endl;
    os << "serverSessionId: " << serverSessionId_ << endl;
    os << "transport:\n";
    os << *transportInfo_;
    return os;
}
