#ifndef _Record_Archive_Manager_h_
#define _Record_Archive_Manager_h_

#include "config.h"
#include "string.h"
#include "RTP_Packet_Sink.h"
#include "RTCP_Packet_Sink.h"
#include "ace/Event_Handler.h"
#include "Stream_Archive_Hash.h"
#include "Record_Session_ID.h"
#include "Voyager_Corba.h"

class Stream_Archive;
class RTSP_Record_Request;

/*
 * Derive from packet sinks so we can grab packets.
 *
 * The Record_Archive_Manager writes the individual data streams into separate
 * files, distinguished by ssrc. For each ssrc, we maintain a record
 * of the formats used in that stream (we don't just mark the format
 * for a stream, as the format can change).
 *
 * We write files in the rtpdump format for easy playback.
 *
 * The file header is
 *
 *	struct timeval start time
 *	unsigned long source (ip address of source)
 *	unsigned short port (port of source)     
 *
 * Each packet is headed by
 *
 *	unsigned short length
 *	unsigned short type (rtp or rtcp)
 *	unsigned long offset (ms since start of recording)
 *	
 * and followed by the original rtp or rtcp packet data.
 *
 * The logic for the writing of stream data is encapsulated in the
 * Stream_Archive object.
 *
 */

class Record_Archive_Manager : public RTP_Packet_Sink ,
			       public RTCP_Packet_Sink , public ACE_Event_Handler
{
public:
    Record_Archive_Manager( VOY_String directory_ , int dbSessionId , Record_Session_ID , VOY_Database_ptr );
    ~Record_Archive_Manager( );

    void terminate( );

    int recording( ) { return recordFlag_; };

    void endOfStream( Stream_Archive* );
    
    const VOY_String& directory( void ) { return directory_; } ;

    void record( RTSP_Record_Request* );

    void enableRecording( );
    void disableRecording( );
    
protected:
    virtual void handlePacket(RTP_Packet::RTP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source,
			       ACE_SOCK_Dgram *source_socket);
    
    virtual void handlePacket(RTCP_Packet::RTCP_Header *packet_header,
			       char *packet, int packet_size,
			       ACE_INET_Addr &packet_source,
			       ACE_SOCK_Dgram *source_socket);

    virtual void handleSubpacket(RTCP_Packet::RTCP_Header *packet_header,
				  char *packet, int packet_size,
				  ACE_INET_Addr &packet_source,
				  ACE_SOCK_Dgram *source_socket);

    virtual int handle_timeout(const ACE_Time_Value &tv, const void *arg);

    VOY_String directory_;

    int recordFlag_;

    short qNeedToMkdir;

    Record_Session_ID sessionId_;

    Stream_Archive_Hash streamHash_;

    VOY_Database_var db_;    

    ACE_Time_Value lastTick_;

private:
    Stream_Archive* createStreamArchive( long ssrc );

    int dbSessionId_;
};

#endif
