#include "config.h"
#include "Record_Manager.h"
#include "RTSP_Setup_Request.h"
#include "RTSP_Record_Request.h"
#include "Record_Network_Transport.h"
#include "Record_Session_Manager.h"
#include "Transport_Manager.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Response.h"
#include "RTSP_Teardown_Request.h"

Record_Manager::Record_Manager(Transport_Manager *tm, VOY_Database_ptr db) :
    transportManager_(tm),
    databaseRef_(db),
    nextSessionId_(1000)
{
}

void Record_Manager::handleSetup(RTSP_Setup_Request *request)
{
    cout << "Record_Manager handling setup " << *request << endl;

    Record_Session_Manager*  recordManager = NULL;

    //
    // Poke at the request to see if we've got a session id yet
    //

    const char *session = request->requestPacket()->headers()["Session"];

    //
    // If the request comes in with a session id already, try
    // to find that session. If it's not there, throw an exception
    // (client evidently sent a bad id. Clients can't choose ids).
    //
    // If there wasn't a session id already, create a new one.
    //

    cout << "Creating session stuff\n";

    Record_Session_ID sessionId;

    if (session != 0 && session[0] != '\0')
    {
	cout << "Looking up session " << session << endl;

	Record_Session_Map::iterator iter;

	sessionId = atoi(session);
	
	iter = recordSessionMap_.find(sessionId);
	if (iter != recordSessionMap_.end())
	{
	    recordManager = (*iter).second;
	    cout << "Found manager " << recordManager << " in map\n";
	}
	else
	{
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
	}
    }
    else
    {
 	Record_Session_ID myid(nextSessionId_++);

	recordManager = new Record_Session_Manager( databaseRef_ , myid , request );

	cout << "Creating new record_session_mgr " << recordManager << "\n";

	recordSessionMap_[myid] = recordManager;
    }

    //
    // Create a transport for this stream
    //

    Record_Network_Transport* netTrans;

    netTrans = transportManager_->setupRecordTransport( request );

    if (netTrans == NULL)
	throw new RTSP_Response(450, "Cannot create record transport",
				request->requestPacket()->sequence());
    else if (recordManager == NULL)
	throw new RTSP_Response(450, "Cannot create record manager",
				request->requestPacket()->sequence());

    else if( recordManager->addSender( request, netTrans ) !=
                 Record_Session_Manager::kSuccess )
	throw new RTSP_Response(450, "Cannot add stream",
				request->requestPacket()->sequence());
}

void Record_Manager::handleRecord(RTSP_Record_Request *request)
{
    // Poke at the request to see if we've got a session id yet
    const char *session = request->requestPacket()->headers()["Session"];

    Record_Session_ID sessionId;
    Record_Session_Manager *recordManager;

    if (session != 0 && session[0] != '\0')
    {
	cout << "Looking up session " << session << endl;

	Record_Session_Map::iterator iter;

	sessionId = atoi(session);
	
	iter = recordSessionMap_.find(sessionId);
	if (iter != recordSessionMap_.end())
	{
	    recordManager = (*iter).second;
	    cout << "Found manager " << recordManager << " in map\n";
	}
	else
	{
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
	}
    }
    else
    {
	throw new RTSP_Response(450, "Invalid session id",
				request->requestPacket()->sequence());
    }

    cout << "recordManager " << recordManager << " gets record request for id ";
    cout << sessionId << endl;

    recordManager->record( request );
}

void
Record_Manager::handleTeardown( RTSP_Teardown_Request* request )
{
    cout << "Record_Manager handling TEARDOWN " << *request << endl;

    const char *session = request->requestPacket()->headers( )["Session"];

    Record_Session_ID               sessionId;
    Record_Session_Manager* recManager;

    if( session != 0  &&  session[0] != '\0')
    {
	cout << "Looking up session " << session << endl;

	Record_Session_Map::iterator iter;

	sessionId = atoi( session );
	
	iter = recordSessionMap_.find( sessionId );

	// Hand the request down to the session manager.  If this is the last stream in the
	// session, delete the Play_Session_Manager
	if( iter != recordSessionMap_.end( ) )
	{
	    recManager = (*iter).second;
	    cout << "Found manager " << recManager << " in map\n";
	    cout << "recManager " << recManager << " gets teardown request for id ";
	    cout << sessionId << endl;

	    short qNoMoreStreams;
	    recManager->teardown( request , transportManager_ , qNoMoreStreams );
	    if( qNoMoreStreams )
	    {
		printf( "Record_Manager::teardown: No more streams in Record_Session_Manager - deleting recManager\n" );
		// Why does deleting the recManager cause db->getStreamPath( )
		// in Stream_Source to seg fault when the next session is setup?
		//delete recManager; 
		// recordSessionMap_.erase( sessionId ); shoulld this be sessionId???
	    }
	    else
		printf( "Record_Manager::teardown: More streams in Record_Session_Manager\n" );
	}
	else
	    throw new RTSP_Response(450, "Invalid session id",
				    request->requestPacket()->sequence());
    }
    else
	throw new RTSP_Response(450, "Invalid session id",
				request->requestPacket()->sequence());    
}

short
Record_Manager::isRecordSession( RTSP_Teardown_Request* request )
{
    short isRecordSession = 0;

    const char *session = request->requestPacket()->headers( )["Session"];

    if( session != 0  &&  session[0] != '\0')
    {
	Record_Session_ID               sessionId;
	sessionId = atoi( session );
	
	Record_Session_Map::iterator iter;
	iter = recordSessionMap_.find( sessionId );

	isRecordSession = ( iter != recordSessionMap_.end( ) );
    }
    
    printf( "Record_Manager::isRecordSession returns %d\n" , isRecordSession );

    return isRecordSession;
}
