
#include "Session_Manager.h"
#include "RTSP_Request_Packet.h"
#include "RTSP_Request.h"
#include "RTSP_Describe_Request.h"
#include "RTSP_Setup_Request.h"
#include "RTSP_Response.h"
#include "RTSP_Engine.h"
#include "Play_Session_Manager.h"
#include "Transport_Manager.h"

Session_Manager::Session_Manager(VOY_Database_var db) : db_(db), transMan_(NULL)
{
}

Session_Manager::Session_Manager() : transMan_( NULL )
{
}

Session_Manager::~Session_Manager( )
{
    if( playSessionMap_.current_size( ) > 0 )
    {
	ACE_Map_Manager<Play_Session_ID,
	    Play_Session_Manager*,
	    VOYAGER_MUTEX>::ITERATOR
	    playSessionIter( playSessionMap_ );
	
	ACE_Map_Manager<Play_Session_ID,
	    Play_Session_Manager*,
	    VOYAGER_MUTEX>::ENTRY*
	    playSessionManagerEntry;
	
	while( playSessionIter.next( playSessionManagerEntry ) )
	    delete playSessionManagerEntry->int_id_;
    }
}

void Session_Manager::addSession(Voyager_Session *s)
{
    printf("binding %d to %x\n", (int)s->id(), s);
    voyagerSessionMap_.bind(s->id(), s);
}

void Session_Manager::handleDescribe(RTSP_Describe_Request *drequest)
{
    RTSP_Request_Packet *request = drequest->requestPacket();
    printf("handling describe %s path=%s seq=%s\n", request->uri(),
	   request->path(), request->sequence());

    Voyager_Session_ID sessionId = drequest->sessionId( );
    char  streamId[ 256 ];
    drequest->streamId( streamId );

    Voyager_Session *session;
    printf("looking up %d\n", (int) sessionId);
    session = this->findVoyagerSession(sessionId);

    if (session == NULL)
    {
	RTSP_Response r(500, "Session not found", request->sequence());
	r.send(request->engine());
    }
    else
    {
	if ( streamId[ 0 ] )
	{
	    session->handleDescribeStream(drequest, streamId);
	}
	else
	{
	    session->handleDescribeSession(drequest);
	}
    }
}

/**
 * Find the session object for this session id. First see if we've already
 * got a copy in memory. If we don't, look it up and load it in from the
 * database.
 **/

Voyager_Session *Session_Manager::findVoyagerSession(Voyager_Session_ID id)
{
    Voyager_Session *session;

    if (voyagerSessionMap_.find(id, session) != -1)
    {
	return session;
    }

    //
    // Otherwise, we need to create one and populate it
    //

    //session = Voyager_Session::loadFromDB(id, voyagerDB_);

    session = Voyager_Session::loadFromDB(id, db_);

    if (session)
    {
	addSession(session);
    }

    return session;
}

void Session_Manager::handleError(RTSP_Request_Packet *request)
{
}

