#ifndef _Stream_Archive_h_
#define _Stream_Archive_h_

#include "config.h"
#include "Voyager_Corba.h"
#include "RTP_Packet.h"
#include "RTCP_Packet.h"
#include "RTP_Dump.h"
#include "Voyager_Corba.h"
#include "ace/INET_Addr.h"
#include "ace/Time_Value.h"

class Record_Archive_Manager;

///////////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
//  Stream_Archive:  Write RTP packets out to a file for a single data stream.  Also
//                              write a db info file that describes the packet data
//
// Modification History
//  16 jan 98:  lisa - adapted from SSRC_Writer
//
///////////////////////////////////////////////////////////////////////////////////////////////////////////////////

/*
 * Write a file for this data stream.
 *
 * Don't actually start writing until an rtp packet comes in.
 *
 * While we're not writing, save the last RTCP packet seen, so that
 * we can start the saved file with a RTCP packet.
 *
 */

class Stream_Archive
{
public:

    Stream_Archive( Record_Archive_Manager* , long ssrc , VOY_Database_ptr , long sessionId ,
		    const char* streamId );
    ~Stream_Archive();

    long ssrc() { return ssrc_; } ;

    virtual void handlePacket( RTP_Packet::RTP_Header* ,  char*packet , int packetSize ,
			       ACE_INET_Addr& packetSource ,  int initialPacket = 0 );
    
    virtual void handlePacket( RTCP_Packet::RTCP_Header* , char*packet , int packetSize ,
			       ACE_INET_Addr& packetSource , int initialPacket = 0 );

    virtual void handleSubpacket( RTCP_Packet::RTCP_Header* ,  char *packet ,
				  int packetSize , ACE_INET_Addr& packetSource);

    void terminate( );

    void tick(const ACE_Time_Value& thisTick ,
	      const ACE_Time_Value& lastTick ,
	      const ACE_Time_Value& delta );

    
protected:

    void openFile( );
    void writePacket( char* packet , int length , int type );
    void writeFileHeader( ACE_INET_Addr& source );
    void writeInitialInfo( );
    void writeFinalInfo( );
    
    VOY_Database_var db_;
    char* streamId_;
    long sessionId_;
    Record_Archive_Manager* archiveMgr_;
    char* fpBuffer_;
    ACE_Time_Value startTime_;
    unsigned long startMs_;
    long ssrc_;
    int lastRtpSeq;
    int bytes_;
    int missedPackets_;
    int packetsSeen_;
    int totalBytes_;
    int totalMissedPackets_;
    int totalPacketsSeen_;
    long fileSize_;

    int haveSynchInfo_;
    double syncNtpTimestamp_;
    int syncRtpTimestamp_;

    int fbHaveCName;  // Flag used to indicate whether we've retrieved the stream's CNAME

    VOY_String streamFilename_;
    FILE* fp_;
    VOY_String infoFilename_;
    FILE* infoFp_;
    char tmpBuf[ 1024];

    /*
     * Storage for the don't-write-the-file-header-until-data-comes-in
     * mechanism.
     */
    int writingOutput_;
    RTCP_Packet savedRtcpPacket_;
    int savedRtcpPacketSize_;
    int haveSavedRtcpPacket_;

    /*
     * Map of RTP packet types that we've seen in this data stream.
     */
    char typesSeen_[128];

    /*
     * Put this in here so we don't have to keep allocating stack space
     * for it.
     */
    RTP_Dump_Packet rdPacket_;

private:
    void setStringStreamKeyValue( char* , char* );
};


#endif /* _Stream_Archive_h */

