#ifndef _Stream_Source_h_
#define _Stream_Source_h_

#include "config.h"
#include "ace/Containers.h"
#include "ace/Event_Handler.h"
#include "Voyager_Corba.h"
#include "RTP_Dump.h"
#include "ace/OS.h"
#include "ace/INET_Addr.h"
#include "RTP_Packet.h"
#include <map.h>


/////////////////////////////////////////////////////////////////////////////
//
// Stream_Source.h
//
// Revision History
//     17 Sep 97 - lisa - original version
//

class RTSP_Setup_Request;
class Play_Stream_Synchronizer;
class Playback_Network_Transport;
class RTSP_Play_Request;
class N_Time;

class Stream_Source : public ACE_Event_Handler
{
public:
    Stream_Source( VOY_String );
    ~Stream_Source( );

    void init( VOY_String );
    virtual int handle_timeout( const ACE_Time_Value& , const void* );
    
    void setup( VOY_Database_ptr , RTSP_Setup_Request* , Play_Stream_Synchronizer* ,
		Playback_Network_Transport* );
    
    void play( RTSP_Play_Request* , ACE_Time_Value , ACE_Time_Value );
    void teardown( void );

    enum TimerType
    {
	kCounter = 0,
	kUpdate = 1,
	kRTCP_Report = 2,
	kMaxTimers = 3
    };

    typedef map< int , long , less<int> > PayloadFreqMap;

private:

    void cancelTimers( void );

    int timerIds_[ kMaxTimers ];
    short openStream( VOY_Database_ptr , RTSP_Setup_Request* );
    short openStreamFile( char* );
    short readHeader( void );
    short readInfoFile( void );
    short handleInfoItem( char* , char* );
    short packetType( char* );
    short calcTime( void );
    double rtpToNtp( unsigned long );
    double rtpToNtp( double );
    double rtpToRelNtp( unsigned long );
    void readPacket( short& , short& , short& );
    double computePlayoutDelay( void );
    void scheduleTimer( double );
    void sendCurrentAndScheduleNextPacket( void );
    void sendPacket( void );
    void terminateStream( void );
    
    int packetBufferLength_;
    RTP_Dump_Packet packet_;
    unsigned long lastPacketTimestamp_;
    unsigned long packetTimestamp_;
    unsigned long timestampFrequency_;
    unsigned long startingTimestamp_;
    unsigned long synchRtpTimestamp_;
    int ptype_;
    double Nsynch_, Nstart_;
    double Rsynch_, Rstart_;
    double F_;
    double synchNtpTimestamp_;
    double playbackStartD_;
    char packetBuffer_[MAX_PACKET];
    short qEof_;
    int wrapAroundTimestampCount_;

    FILE* streamFile_;
    char* fileBuffer_;
    char  streamSpec_[1024];

    RTP_Dump_Header header_;
    ACE_Time_Value startTime_;
    ACE_INET_Addr originalSource_;

    Play_Stream_Synchronizer* synch_;
    Playback_Network_Transport* netTrans_;

    PayloadFreqMap payloadFreqMap_;
    FILE*          payFreqFile_;
};
    
#endif
