/*
 * Shark Distributed Multi-Media System
 *
 * Licensed Materials-Property of IBM
 * (C) Copyright IBM Corp. 1995
 * All rights reserved.
 * US Government Users Restricted Rights -
 * Use, duplication of Disclosure restricted
 * by GSA ADP Schedule Contract with IBM Corp.
 */

/*
 * $Id: tsfattr.h,v 1.1 1998/02/27 02:09:32 childers Exp $
 */

#ifndef _h_tsfattr
#define _h_tsfattr

/* defines for command parameter in tsfattr() */
#define SET_DFLT_FILE_RATE     4
#define GET_RATE               5
#define SET_RATE               6
#define REL_RSRV               7
#define ACT_RSRV               8
#define INACT_RSRV             9
#define SET_BUFFERING          10
#define GET_BUFFERING          11

#define GET_DFLT_FILE_RATE     15

#define LLSEEK                 16
#define LLSEEK_SET             16  /* LLSEEK + SEEK_SET (0) */
#define LLSEEK_CUR             17  /* LLSEEK + SEEK_CUR (1) */
#define LLSEEK_END             18  /* LLSEEK + SEEK_END (2) */

#define GET_REPL_FACTORS       20
#define SET_REPL_FACTORS       21


#define REVOKE_TOKEN       99  /* for testing only */


/* Structure used to pass replication factors to/from tsfattr
   for GET_REPL_FACTORS and SET_REPL_FACTORS */
struct tsrep
{
  int dataReplicas;
  int maxDataReplicas;
  int metadataReplicas;
  int maxMetadataReplicas;
};

/* Structure used to receive additional information regarding
   SET_REPL_FACTORS operation failures. */
typedef struct
{
  int reason;               /* reason request failed */
  int value1;               /* optional value depending upon reason */
  int value2;               /* optional value depending upon reason */
} tsfattrReasonCodeInfo;

/* ************************************************************************* */
/* Values that may be returned by tsfattr() in tsfattrReasonCodeInfo.reason. */

/* No reason information was returned. */
#define TSFATTR_NONE                       0

/* MetadataReplicas is out of range.
   Value1 and value2 contain the valid lower and upper range boundaries. */
#define TSFATTR_METADATA_REPLICAS_RANGE    1

/* MaxMetadataReplicas is out of range.
   Value1 and value2 contain the valid lower and upper range boundaries. */
#define TSFATTR_MAXMETADATA_REPLICAS_RANGE 2

/* DataReplicas is out of range.
   Value1 and value2 contain the valid lower and upper range boundaries. */
#define TSFATTR_DATA_REPLICAS_RANGE        3

/* MaxDataReplicas is out of range.
   Value1 and value2 contain the valid lower and upper range boundaries. */
#define TSFATTR_MAXDATA_REPLICAS_RANGE     4

/* An attempt to change maxMetadataReplicas or maxDataReplicas or both
   was made on a file that is not empty. */
#define TSFATTR_FILE_NOT_EMPTY             5

/* MetadataReplicas or dataReplicas or both exceed the number of failure groups.
   Value1 contains the maximum number of metadata failure groups.
   Value2 contains the maximum number of data failure groups. */
#define TSFATTR_REPLICAS_EXCEED_FGMAX      6
/* ************************************************************************* */

/* Definitions for TigerShark file attribute function */


#ifdef __cplusplus
extern "C" {
#endif

extern int tsfattr(int fileDesc,  /* Open file descriptor */
                   int command,   /* Control function to be performed */
                   void *argP,    /* Additional info required by function */
                   tsfattrReasonCodeInfo *rCodeP); /* More returned info*/

#ifdef __cplusplus
}
#endif

#endif /* _h_tsfattr */

